--[[
Name: LibBanzai-2.0
Revision: $Revision: 59737 $
Author(s): Rabbit (rabbit.magtheridon@gmail.com), maia
Documentation: http://www.wowace.com/index.php/Banzai-2.0_API_Documentation
SVN: http://svn.wowace.com/wowace/trunk/BanzaiLib/Banzai-2.0
Description: Aggro notification library.
Dependencies: LibStub
]]

-------------------------------------------------------------------------------
-- Locals
-------------------------------------------------------------------------------

local MAJOR_VERSION = "LibBanzai-2.0"
local MINOR_VERSION = "$Revision: 59737 $"

if not LibStub then error(MAJOR_VERSION .. " requires LibStub.") end
local lib, oldLib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

lib.callbacks = oldLib and oldLib.callbacks or {}
local callbacks = lib.callbacks
lib.frame = oldLib and oldLib.frame or CreateFrame("Frame")
local frame = lib.frame

local _G = _G
local table_insert = table.insert
local UnitExists = _G.UnitExists
local UnitName = _G.UnitName
local UnitCanAttack = _G.UnitCanAttack
local GetNumRaidMembers = _G.GetNumRaidMembers
local GetNumPartyMembers = _G.GetNumPartyMembers
local unpack = _G.unpack
local type = _G.type
local assert = _G.assert

-------------------------------------------------------------------------------
-- Local heap
-------------------------------------------------------------------------------

local new, del
do
	local cache = setmetatable({},{__mode='k'})
	function new()
		local t = next(cache)
		if t then
			cache[t] = nil
			return t
		else
			return {}
		end
	end
	function del(t)
		for k in pairs(t) do
			t[k] = nil
		end
		cache[t] = true
		return nil
	end
end

-------------------------------------------------------------------------------
-- Roster
-------------------------------------------------------------------------------

local raidUnits = setmetatable({}, {__index =
	function(self, key)
		self[key] = "raid" .. key
		return self[key]
	end
})
local partyUnits = {"party1","party2","party3","party4","party5"}
local roster = {}

local function addUnit(unit)
	if not UnitExists(unit) then return end
	local name = UnitName(unit)
	if not roster[name] then roster[name] = new() end
	table_insert(roster[name], unit)
end

local function updateRoster()
	for k in pairs(roster) do roster[k] = del(roster[k]) end
	addUnit("player")
	for i = 1, GetNumRaidMembers() do addUnit(raidUnits[i]) end
	for i = 1, GetNumPartyMembers() do addUnit(partyUnits[i]) end
end

-------------------------------------------------------------------------------
-- Banzai
-------------------------------------------------------------------------------

local targets = setmetatable({}, {__index =
	function(self, key)
		self[key] = key .. "target"
		return self[key]
	end
})

local aggro = {}
local banzai = {}

local total = 0
local function updateBanzai(_, elapsed)
	total = total + elapsed
	if total > 1 then
		for name, units in pairs(roster) do
			local unit = units[1]
			local targetId = targets[unit]
			if UnitExists(targetId) then
				local ttId = targets[targetId]
				if UnitExists(ttId) and UnitCanAttack(ttId, targetId) then
					for n, u in pairs(roster) do
						if UnitIsUnit(u[1], ttId) and (not banzai[n] or banzai[n] < 20) then
							banzai[n] = (banzai[n] or 0) + 10
							break
						end
					end
				end
			end
		end
		for name, units in pairs(roster) do
			if banzai[name] >= 5 then banzai[name] = banzai[name] - 5 end
			if banzai[name] > 15 then
				if not aggro[name] then
					aggro[name] = true
					for i, v in ipairs(callbacks) do
						v(1, name, unpack(units))
					end
				end
			elseif aggro[name] then
				aggro[name] = nil
				for i, v in ipairs(callbacks) do
					v(0, name, unpack(units))
				end
			end
		end
		total = 0
	end
end

-------------------------------------------------------------------------------
-- Starting and stopping
-------------------------------------------------------------------------------

local running = nil
local function start()
	if running then return end
	updateRoster()
	frame:SetScript("OnUpdate", updateBanzai)
	frame:SetScript("OnEvent", updateRoster)
	frame:RegisterEvent("RAID_ROSTER_UPDATE")
	frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	running = true
end

local function stop()
	if not running then return end
	frame:SetScript("OnUpdate", nil)
	frame:SetScript("OnEvent", nil)
	frame:UnregisterAllEvents()
	running = nil
end

-------------------------------------------------------------------------------
-- API
-------------------------------------------------------------------------------

function lib:IsRunning() return running end
function lib:GetUnitAggroByUnitName(name) return aggro[name] end
function lib:GetUnitAggroByUnitId(unit)
	if not UnitExists(unit) then return end
	return aggro[UnitName(unit)]
end

function lib:RegisterCallback(func)
	assert(type(func) == "function", ("Bad argument to :RegisterCallback, function expected, got %q."):format(type(func)))
	table_insert(callbacks, func)
	start()
end

function lib:UnregisterCallback(func)
	assert(type(func) == "function", ("Bad argument to :UnregisterCallback, function expected, got %q."):format(type(func)))
	for i, v in ipairs(callbacks) do
		if v == func then
			table.remove(callbacks, i)
			return
		end
	end
	if #callbacks == 0 then stop() end
end

-------------------------------------------------------------------------------
-- Initialization
-------------------------------------------------------------------------------

frame:SetScript("OnUpdate", nil)
frame:SetScript("OnEvent", nil)
frame:UnregisterAllEvents()
if #callbacks > 0 then start() end

